import os
import sys
import subprocess
import argparse

from pathlib import Path

dirs = [
    'algorithms',
    'misc',
#    'contests',
#    'problems'
]

parser = argparse.ArgumentParser()
parser.add_argument('--header', action='store', type=str, help='The text to parse.')
parser.add_argument('--output', action='store', type=str, help='The text to parse.')
args = parser.parse_args()

output = open(args.output, 'w')

# Read Latex header
with open(args.header) as f:
    data = f.readlines()

# Print header to output
for i in data:
    output.write(i)

last_sections = [None] * 2

for di in dirs: 
    path_list = Path(di).glob('**/*.cpp')

    for path in path_list:
        file_name = str(path)
        sections = file_name.replace('_', '\_').split('/')

        # Sections[0] is [algorithms, contests, problems, misc] (sections)
        if sections[0] != last_sections[0]:
            output.write('\\newpage\n')
            output.write('\\section{' + sections[0].capitalize() + '}\n')

        # Sections[1] is name of constest or category of algorithm (subsections)
        if sections[1] != last_sections[1]:
            output.write('\\subsection{' + sections[1].capitalize() + '}\n')

        # Parse source code
        with open(file_name) as f:
            source = f.readlines()

        # Separate into comment and code, and define title
        title = ""
        in_comment = False
        code, comment = [], []
        for line in source:
            if '/**' == line[0:3]:
                in_comment = True

            if in_comment:
                if len(title) == 0 and len(comment) == 1:
                    title = line[3:]
                comment.append(line)
            else:
                code.append(line)

            if '*/' in line:
                in_comment = False

        in_ctime = False
        ctime, cspace = [], []
        for line in comment:
            if 'Complexity (time)' in line:
                in_ctime = True
            if 'Complexity (space)' in line:
                in_ctime = False

            if in_ctime:
                ctime.append(line)



        if len(sections) > 2:
            output.write('\\subsubsection{' + title + '}\n')

        # Remove first \n after header comment
        code = code[1:]

        #output.write('\\lstinputlisting[style=customcpp]{' + file_name + '}\n')
        output.write('\\begin{lstlisting}[style=customcpp]\n')
        for i in code:
            output.write(i)
        output.write('\\end{lstlisting}\n')
        output.write('\\\n')
        last_sections = sections

output.write('\\end{document}')
